import { Component, EventEmitter, Input, OnInit, Output, ViewChild } from '@angular/core';
import { UntypedFormBuilder, UntypedFormGroup, Validators } from '@angular/forms';
import { Router, ActivatedRoute } from '@angular/router';
import { PublicConfig } from 'app/core/models/public-config';
import { AppStateService } from 'app/core/services/app.state.service';
import { BaseComponent } from 'app/shared/components/base/base.component';
import { BackendError } from 'app/shared/models/backend-error';
import { NotificationService } from 'app/shared/services/notification.service';
import { takeUntil } from 'rxjs/operators';
import { CreateCourseFileDto } from '../../dto/create-course-file.dto';
import { Course } from '../../models/course';
import { CourseFile } from '../../models/course-file';
import { CourseFileService } from '../../services/course-file.service';

@Component({
  selector: 'ngx-course-file-upload',
  templateUrl: './course-file-upload.component.html',
  styleUrls: ['./course-file-upload.component.scss']
})
export class CourseFileUploadComponent extends BaseComponent implements OnInit {
  @Input() course: Course;
  @Output() fileUploaded = new EventEmitter<CourseFile>();

  uploading: boolean;
  form: UntypedFormGroup;
  file: File;

  config: PublicConfig;

  constructor(
    private courseFileService: CourseFileService,
    private appStateService: AppStateService,
    private router: Router,
    private route: ActivatedRoute,
    private notificationService: NotificationService,
    private fb: UntypedFormBuilder) {
    super();
    this.config = appStateService.config;
  }

  ngOnInit(): void {
    this.createForm();
  }

  createForm() {
    this.form = this.fb.group({
      file: ['', Validators.required],
      description: ['']
    });
  }

  handleFile(file: File) {
    this.file = file;
    this.form.patchValue({
      file: file
    });
  }
  
  save() {
    this.uploading = true;

    var dto: CreateCourseFileDto = {
      courseId: this.course.id,
      description: this.form.get('description').value,
    }

    this.courseFileService.create(dto, this.form.get('file').value)
      .pipe(takeUntil(this.unsubscribe))
      .subscribe(task => {
        this.notificationService.showSuccess("Datei erfolgreich hochgeladen");
        this.fileUploaded.emit(task);
        this.uploading = false;
        this.file = null;
        this.form.reset();
      }, (error: any) => {
        this.uploading = false;

        if (BackendError.isBackendError(error)) {
          this.notificationService.showError(
            error.format(
              "Fehler beim Hochladen der Datei" 
            )
          );
        } else {
          throw error;
        }
      });
  }
}
