import { Component, OnInit } from '@angular/core';
import { BaseComponent } from 'app/shared/components/base/base.component';
import { DashboardState } from '../../models/dashboard-state';
import { DashboardService } from '../../services/dashboard.service';
import { takeUntil, flatMap, map } from 'rxjs/operators';
import { Weekday } from 'app/features/courses/models/weekday';
import { AuthService } from 'app/core/services/auth.service';
import { combineLatest } from 'rxjs';
import { AppStateService } from 'app/core/services/app.state.service';
import { Term } from 'app/features/modules/models/term';
import { BackendError } from 'app/shared/models/backend-error';
import { SubatoPrincipal } from 'app/core/models/principal/subato-principal';
import { Role } from 'app/core/models/role';

@Component({
  selector: 'ngx-dashboard',
  templateUrl: './dashboard.component.html',
  styleUrls: ['./dashboard.component.scss']
})
export class DashboardComponent extends BaseComponent implements OnInit {
  loading: boolean = true;
  dashboard: DashboardState;
  Weekday = Weekday;
  principal: SubatoPrincipal;
  term: Term | null;

  error: BackendError;
  Role = Role;

  constructor(
    private authService: AuthService,
    private appStateService: AppStateService,
    private dashboardService: DashboardService) {
    super();
  }

  ngOnInit(): void {
    combineLatest(this.appStateService.term$, this.authService.principal$)
      .pipe(
        takeUntil(this.unsubscribe),
        flatMap(([term, principal]) => {
          this.loading = true;
          this.principal = principal;
          this.term = term;
          return this.dashboardService.getDashboard(term?.id);
        })
      ).subscribe(dashboard => {

        this.dashboard = dashboard;
        this.loading = false;
      }, error => {
        if (BackendError.isBackendError(error)) {
          this.error = error;
          this.loading = false;
        } else {
          throw error;
        }
      });
  }
}
