import { HttpClient, HttpParams } from "@angular/common/http";
import { Injectable } from "@angular/core";
import { BackendError } from "app/shared/models/backend-error";
import { BaseService } from "app/shared/services/base.service";
import { environment } from "environments/environment";
import { Observable } from "rxjs";
import { catchError, map } from "rxjs/operators";
import { DashboardState } from "../models/dashboard-state";

@Injectable({
    providedIn: 'root'
})
export class DashboardService extends BaseService {
    constructor(private http: HttpClient) {
        super();
    }

    getDashboard(termId: number): Observable<DashboardState> {
        let params = new HttpParams();

        if(termId != null) {
            params = params.set('termId', termId)
        }
        
        return this.http.get<DashboardState>(`${environment.backendUrl}/dashboard`, {
            params: params
        }).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(dashboard => new DashboardState(dashboard))
        );
    }
}