import { AssessmentProgress } from "./assessment-progress";
import { Submission } from "./submission";

export class BinaryAssessmentProgress extends AssessmentProgress {
    passedTotal: number;
    notPassedTotal: number;
    undecidedTotal: number;
    total: number;

    constructor(dto: any) {
        super(dto);
        this.passedTotal = dto.passedTotal;
        this.notPassedTotal = dto.notPassedTotal;
        this.undecidedTotal = dto.undecidedTotal;
        this.total = dto.total;
    }

    update(submissions: Submission[]): BinaryAssessmentProgress {
        return new BinaryAssessmentProgress({
            passedTotal: submissions.filter(s => s.passed == true).length,
            notPassedTotal: submissions.filter(s => s.passed == false).length,
            undecidedTotal: submissions.filter(s => s.passed == null).length,
            total: submissions.length,
        });
    }

    get progress() {
        return this.totalBinary != 0 ? this.decided / this.totalBinary * 100 : 100;
    }

    get totalBinary() {
        return this.notPassedTotal + this.passedTotal + this.undecidedTotal;
    }

    get decided() {
        return this.totalBinary - this.undecidedTotal;
    }
}