import { HttpClient, HttpParams } from "@angular/common/http";
import { Injectable } from "@angular/core";
import { BaseService } from "app/shared/services/base.service";
import { environment } from "environments/environment";
import { Observable } from "rxjs";
import { catchError, map } from "rxjs/operators";
import { BackendError } from "app/shared/models/backend-error";
import { Misconception } from "../models/misconception";

@Injectable({
    providedIn: 'root'
})
export class MisconceptionService extends BaseService {
    constructor(private http: HttpClient) {
        super();
    }

    getById(id: string): Observable<Misconception> {
        return this.http.get<Misconception>(`${environment.backendUrl}/misconception/${id}`).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(misconception => new Misconception(misconception))
        );
    }

    getAll(): Observable<Misconception[]> {
        return this.http.get<Misconception[]>(`${environment.backendUrl}/misconception`).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(misconceptions => misconceptions.map(misconception => new Misconception(misconception)))
        )
    }
}