import { Injectable, ViewContainerRef } from "@angular/core";
import { Solution } from "app/features/solutions/models/solution";
import { SolutionService } from "app/features/solutions/services/solution.service";
import { BaseComponent } from "app/shared/components/base/base.component";
import { BackendError } from "app/shared/models/backend-error";
import { FileTree } from "app/shared/models/file-tree";
import { NotificationService } from "app/shared/services/notification.service";
import { Observable } from "rxjs";
import { catchError, map, takeUntil } from "rxjs/operators";
import { PdfViewerComponent } from "./pdf-viewer.component";

@Injectable({
    providedIn: 'root'
})
export class PdfViewerFactory extends BaseComponent {
    constructor(
        private solutionService: SolutionService,
        private notificationService: NotificationService
    ) {
        super();
    }

    createViewer(solution: Solution, tree: FileTree, containerRef: ViewContainerRef): Observable<void> {
        return this.solutionService.getFileAsUrl(solution.id, tree)
            .pipe(
                takeUntil(this.unsubscribe),
                catchError((error: any) => {
                    if (BackendError.isBackendError(error)) {
                        this.notificationService.showError(
                            error.format(
                                "Fehler beim Herunterladen der Lösung",
                            )
                        );
                    }

                    throw error;
                }),
                map(([url, fileName]) => {
                    const componentRef = containerRef.createComponent<PdfViewerComponent>(PdfViewerComponent);
                    componentRef.instance.url = url;
                }),
            );
    }
}