import { Component, ContentChild, Input, OnInit, TemplateRef } from '@angular/core';
import { SubatoPrincipal } from 'app/core/models/principal/subato-principal';
import { AuthService } from 'app/core/services/auth.service';
import { BackendError } from 'app/shared/models/backend-error';
import { BaseComponent } from '../base/base.component';
import { ErrorContainerConfig } from '../error-container/error-container.component';
import { takeUntil, flatMap, filter, catchError } from 'rxjs/operators';

@Component({
  selector: 'ngx-page-container',
  templateUrl: './page-container.component.html',
  styleUrls: ['./page-container.component.scss']
})
export class PageContainerComponent extends BaseComponent implements OnInit {
  @Input() loading: boolean = false;
  @Input() error: BackendError | null = null;
  @Input() errorConfig: ErrorContainerConfig | null = null;
  @ContentChild(TemplateRef) bodyTpl;

  principal: SubatoPrincipal | null;

  constructor(private authService: AuthService) {
    super();
  }

  ngOnInit(): void {
    this.authService.principal$
      .pipe(takeUntil(this.unsubscribe))
      .subscribe(principal => this.principal = principal);
  }
}
