package de.hsrm.sls.subato.auth.policy;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.security.access.AccessDeniedException;
import org.springframework.stereotype.Component;

import java.util.List;

@Component
public class PolicyEvaluator {
    private static Logger logger = LoggerFactory.getLogger(PolicyEvaluator.class);

    @Autowired
    private List<PolicyRule> rules;

    public boolean check(CheckPolicyRequest request) {
        for(PolicyRule rule : rules) {
            try {
                if(rule.check(request)) {
                    return true;
                }
            } catch(Exception ex) {
                logger.info("An error occurred while evaluating PolicyRule.", ex);
            }
        }


        return false;
    }

    /**
     * For usage in controller body
     *
     * policy.checkAndRaise(CheckPolicyRequest.fromPrincipal(id, Permission.MI_UPDATE));
     * @param request
     */
    public void checkAndRaise(CheckPolicyRequest request) {
        var access = check(request);
        if(!access) {
            throw new AccessDeniedException("");
        }
    }

    public Specification checkAsSpec(CheckPolicyRequest request) {
        var totalSpec = Specification.where(null);
        for(PolicyRule rule : rules) {
            try {
                var spec = rule.checkAsSpec(request);
                totalSpec = totalSpec.or(spec);
            } catch(Exception ex) {
                logger.info("An error occurred while evaluating PolicyRule.", ex);
            }
        }

        return totalSpec;
    }
}
