package de.hsrm.sls.subato.modules.progmodule;

import de.hsrm.sls.subato.modules.term.TermNotFoundException;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.responses.ApiResponses;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.server.ResponseStatusException;

import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

import static org.springframework.http.HttpStatus.BAD_REQUEST;

@RestController
@Tag(name = "Modulverwaltung")
public class ProgramModuleController {
    @Autowired
    private ProgramModuleService programModuleService;

    @Autowired
    private ProgramModuleMapper programModuleMapper;

    @GetMapping("/module")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "400", description = TermNotFoundException.REASON),
    })
    List<ProgramModuleSummaryDto> getAll(@RequestParam Optional<Long> termId, @RequestParam("q") Optional<String> query) {
        try {
            var modules = programModuleService.getAllForTerm(termId.orElse(null), query.orElse(null));
            return modules.stream().map(i -> programModuleMapper.mapSummary(i)).collect(Collectors.toList());
        } catch(TermNotFoundException ex) {
            throw new ResponseStatusException(BAD_REQUEST, TermNotFoundException.REASON);
        }
    }
}
