package de.hsrm.sls.subato.seed;

import de.hsrm.sls.subato.aor.term.TermDataImportManager;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.File;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;

@Service
@Transactional
public class TermDataSeedService {
    private static Logger logger = LoggerFactory.getLogger(TermDataSeedService.class);

    @Autowired
    private TermDataImportManager termDataImportManager;

    public void seedTermData(File seedDir) {
        try {

            var coreDataDir = seedDir.toPath().resolve("term-data").toFile();
            if(coreDataDir.exists()) {
                logger.debug("Seeding term data...");

                for(var file : coreDataDir.listFiles()) {
                    logger.debug("Processing %s...".formatted(file.toPath().toString()));
                    var json = Files.readString(file.toPath(), StandardCharsets.UTF_8);
                    termDataImportManager.importFromJson(json);
                }
            }
        } catch (IOException e) {
            throw new SeedingException(e);
        }
    }
}
