package de.hsrm.sls.subato.solutions.comment;

import de.hsrm.sls.subato.auth.policy.CheckPolicyRequest;
import de.hsrm.sls.subato.auth.policy.PolicyEvaluator;
import de.hsrm.sls.subato.auth.policy.solutions.SolutionsAction;
import de.hsrm.sls.subato.auth.principal.PrincipalService;
import de.hsrm.sls.subato.solutions.solution.SolutionService;
import de.hsrm.sls.subato.exercises.submission.SubmissionNotFoundException;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.server.ResponseStatusException;

import static org.springframework.http.HttpStatus.BAD_REQUEST;
import static org.springframework.http.HttpStatus.NOT_FOUND;

@RestController
@Tag(name = "Lösungsverwaltung")
public class SolutionCommentController {
    @Autowired
    private SolutionCommentMapper solutionCommentMapper;

    @Autowired
    private SolutionCommentService solutionCommentService;

    @Autowired
    private SolutionService solutionService;

    @Autowired
    private PrincipalService principalService;

    @Autowired
    private PolicyEvaluator policy;

    @PostMapping("/solution_comment")
    SolutionCommentDto create(@RequestBody CreateSolutionCommentDto dto) {
        try {
            var solution = solutionService.findById(dto.getSolutionId());
            policy.checkAndRaise(CheckPolicyRequest.fromPrincipal(solution, SolutionsAction.SOLUTION_COMMENT_CREATE));

            var comment = solutionCommentService.create(dto, solution, principalService.getPrincipal());
            return solutionCommentMapper.map(comment);
        } catch(SubmissionNotFoundException ex) {
            throw new ResponseStatusException(BAD_REQUEST, "Die Abgabe existiert nicht");
        }
    }

    @DeleteMapping("/solution_comment/{id}")
    void delete(@PathVariable Long id) {
        try {
            var comment = solutionCommentService.findById(id);
            policy.checkAndRaise(CheckPolicyRequest.fromPrincipal(comment, SolutionsAction.SOLUTION_COMMENT_DELETE));

            solutionCommentService.delete(comment);
        } catch(SolutionCommentNotFoundException ex) {
            throw new ResponseStatusException(NOT_FOUND, "Der Kommentar existiert nicht");
        }
    }
}
