package de.hsrm.sls.subato.tasks.pool.sync;

import de.hsrm.sls.subato.tasks.pool.TaskPoolService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.CommandLineRunner;
import org.springframework.stereotype.Component;

/**
 * Responsible for manually synchronizing all task pools linked to git repositories
 *
 * for starting the synchronization, run:
 * gradle bootRun --args='sync:pools'
 */
@Component
public class TaskPoolSyncRunner implements CommandLineRunner {
    @Autowired
    TaskPoolSyncService syncService;

    @Autowired
    TaskPoolService taskPoolService;

    private static Logger logger = LoggerFactory.getLogger(TaskPoolSyncRunner.class);

    @Override
    public void run(String... args) {
        if (args.length < 1 || !args[0].equals("sync:pools")) return;

        try {
            var pools = taskPoolService.getAllWithRepository();
            for(var pool : pools) {
                syncService.syncPool(pool, true);
            }

            logger.info("Finished Task Pool sync");
        } catch(Exception e) {
            logger.error("Task Pool sync failed", e);
        }

        System.exit(0);
    }
}
