package de.hsrm.sls.subato.tasks.task;

import com.google.common.base.Strings;
import de.hsrm.sls.subato.exercises.taskinstance.TaskInstance;
import de.hsrm.sls.subato.shared.config.SubatoConfig;
import jakarta.servlet.http.HttpServletRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.nio.file.Path;
import java.util.regex.Pattern;

/**
 * Fix relative urls to assets in the html content of the task,
 * so that the url points to our endpoint, serving the asset
 * @see TaskController#downloadAsset
 */
@Service
public class TaskReferenceRewriter {
    @Autowired
    private SubatoConfig subatoConfig;

    String getBaseUrl(HttpServletRequest request) {
        var hostname = Strings.isNullOrEmpty(subatoConfig.getPublicHostname()) ? request.getServerName() : subatoConfig.getPublicHostname();

        return ""
                + request.getScheme() + "://"
                + hostname
                + ":" + request.getServerPort();
    }

    private String rewriteUrls(String content, String path, HttpServletRequest request) {
        var url = "%s/%s".formatted(getBaseUrl(request), path);

        var matcher = Pattern.compile("src=\"(\\S+)\"").matcher(content);
        return matcher.replaceAll(r -> {
            var matchedUrl = r.group(1);
            var correctedUrl = matchedUrl;

            if(!matchedUrl.startsWith("/") && !matchedUrl.startsWith("https://") && !matchedUrl.startsWith("http://")) {
                correctedUrl = "%s%s".formatted(url, matchedUrl);
            }

            return "src=\"%s\"".formatted(correctedUrl);
        });
    }

    public String rewrite(Task task, HttpServletRequest request) {
        if(task.isArchived()) {
            throw new TaskArchivedException();
        }

        var taskAssetPath = Path.of(
                "task",
                String.valueOf(task.getId()),
                "asset?path=").toString();
        var spec = task.getSpec();
        return rewriteUrls(spec.getText(), taskAssetPath, request);
    }

    public String rewrite(TaskInstance taskInstance, HttpServletRequest request) {
        if(taskInstance.getTask() != null) {
            if(taskInstance.getTask().isArchived()) {
                throw new TaskArchivedException();
            }

            var taskAssetPath = Path.of(
                    "exercise",
                    String.valueOf(taskInstance.getExercise().getId()),
                    "task",
                    String.valueOf(taskInstance.getTask().getId()), "asset?path=").toString();

            var spec = taskInstance.getTask().getSpec();
            return rewriteUrls(spec.getText(), taskAssetPath, request);
        }

        return "";
    }
}