package de.hsrm.sls.subato.tasks.pool.sync;

import de.hsrm.sls.subato.utils.PrepareSubatoDataDir;
import de.hsrm.sls.subato.utils.SubatoTmpDirCleanup;
import de.hsrm.sls.subato.Tags;
import de.hsrm.sls.subato.tasks.pool.TaskPoolRepository;
import org.junit.Assert;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Tag;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.jdbc.AutoConfigureTestDatabase;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.annotation.DirtiesContext;
import org.springframework.transaction.annotation.Transactional;

/**
 * Integration test for importing tasks from the STEF example task pool
 *
 * Remote dependencies:
 * - GitLab (access to repository of the task pool)
 * - Eva (resolving evaluators of tasks)
 */
@SpringBootTest
@DirtiesContext(classMode = DirtiesContext.ClassMode.BEFORE_EACH_TEST_METHOD)
@AutoConfigureTestDatabase(replace = AutoConfigureTestDatabase.Replace.ANY)
@Tag(Tags.INTEGRATION)
@Tag(Tags.EXTERNAL_DEPENDENCY)
@Transactional
public class TaskPoolSyncServiceTest {
    @Autowired
    TaskPoolSyncService syncService;

    @Autowired
    TaskPoolRepository poolRepository;

    @BeforeEach
    public void beforeEach(@Autowired SubatoTmpDirCleanup tmpDirCleanup) {
        tmpDirCleanup.exec();
    }

    @BeforeAll
    public static void beforeAll(@Autowired PrepareSubatoDataDir dataDirCleanup, @Autowired SubatoTmpDirCleanup tmpDirCleanup) {
        dataDirCleanup.exec();
        tmpDirCleanup.exec();
    }

    @Test
    public void shouldImportStefExamplesFromPublicRepo() {
        var pool = poolRepository.findById(1L).get();
        var result = syncService.syncPool(pool, false);
        Assert.assertEquals(0, result.getProblems());
        Assert.assertEquals(result.getTotalTaskDirectories(), result.getImportedTasks());
        Assert.assertEquals(0, result.getArchivedTasks());
    }

    @Test
    public void shouldSkipIfNoChangesInRepo() {
        var pool = poolRepository.findById(1L).get();
        var result = syncService.syncPool(pool, false);
        Assert.assertEquals(0, result.getProblems());
        Assert.assertEquals(result.getTotalTaskDirectories(), result.getImportedTasks());
        Assert.assertEquals(0, result.getArchivedTasks());

        result = syncService.syncPool(pool, false);
        Assert.assertNull(result);
    }
}
